==> spread-5.7.0.72-py3-none-any.whl: spread_dali/payloads.py <==
from datetime import datetime

from spread_core.tools.utils import bytes2str
from . import command, frame, address, gear, device


class Payload:
    mark = 'Payload'

    def __init__(self, _frame: bytes):
        self.frame = _frame

    def __repr__(self):
        return f'[{self.name}]: {bytes2str(self.frame)}'

    def to_dict(self):
        return dict(
            time=self.time,
            hex=bytes2str(self.frame),
            name=self.name
        )

    @property
    def time(self):
        time = datetime.now()
        return time.strftime('%H:%M:%S') + f'.{str(int(time.microsecond / 1000)).ljust(3, "0")}'

    @property
    def name(self) -> str: return self.__class__.__name__

    @property
    def desc(self) -> str: return ''


class DaliPayload(Payload):
    @staticmethod
    def of(_frame: bytes, query_class=None):
        if len(_frame) < 2:
            return DaliResponse(query_class, _frame)
        elif len(_frame) == 2:
            return DaliCommand(_frame)
        elif len(_frame) == 3:
            if _frame[0] & 1 == 0:
                return Dali2Event(_frame)
            else:
                return Dali2Command(_frame)

        raise ValueError('frame size more than 3')

    @property
    def command(self) -> command.Command:
        if not hasattr(self, '_command'):
            self._command = command.from_frame(frame.ForwardFrame(8 * len(self.frame), self.frame))
        return self._command

    @property
    def has_address(self):
        return False

    @property
    def has_instance(self):
        return False

    @property
    def address(self):
        raise ValueError("No address")

    @property
    def instance(self):
        raise ValueError("No instance")

    @staticmethod
    def pack_instance(inst) -> str:
        if isinstance(inst, address.InstanceNumber):
            return f'IN{inst._value}'
        elif isinstance(inst, address.InstanceType):
            return f'IT{inst._value}'
        elif isinstance(inst, address.InstanceGroup):
            return f'IG{inst._value}'
        elif isinstance(inst, address.InstanceBroadcast):
            return 'IB'
        elif isinstance(inst, address.FeatureInstanceType):
            return f'FT{inst._value}'
        elif isinstance(inst, address.FeatureInstanceGroup):
            return f'FG{inst._value}'
        elif isinstance(inst, address.FeatureInstanceNumber):
            return f'FN{inst._value}'
        elif isinstance(inst, address.FeatureDevice):
            return f'FD{inst._value}'
        elif isinstance(inst, address.FeatureInstanceBroadcast):
            return f'FB'
        return 'X3'

    @staticmethod
    def pack_address(addr) -> str:
        if isinstance(addr, address.Short):
            return f'D{addr.address}'
        elif isinstance(addr, address.Group):
            return f'G{addr.group}'
        elif isinstance(addr, address.Broadcast):
            return 'B'
        elif isinstance(addr, address.BroadcastUnaddressed):
            return 'BU'
        return 'X3'


class DaliCommand(DaliPayload):
    mark = 'dali'

    @property
    def has_address(self):
        return isinstance(self.command, (gear.general._StandardCommand, gear.general.DAPC))

    @property
    def has_params(self):
        if isinstance(self.command, gear.general.DAPC):
            return True
        return isinstance(self.command, gear.general._StandardCommand) and self.command._hasparam

    @property
    def params(self):
        if isinstance(self.command, gear.general.DAPC):
            return [self.command.power]
        return [self.command.param]

    @property
    def address(self):
        return self.pack_address(self.command.destination)

    @property
    def desc(self) -> str:
        if self.has_params:
            return ' '.join([self.name, ' '.join([str(p) for p in self.params])])
        return self.name

    @property
    def name(self) -> str:
        return self.command.__class__.__name__


class Dali2Command(DaliCommand):
    mark = 'dali2'

    @property
    def has_address(self):
        return isinstance(self.command, device.general._DeviceCommand) and not isinstance(self.command, device.general._SpecialDeviceCommand)

    @property
    def has_instance(self):
        return isinstance(self.command, device.general._StandardInstanceCommand)

    @property
    def has_params(self):
        return isinstance(self.command, (
            device.general._SpecialDeviceCommandOneParam, device.general._SpecialDeviceCommandTwoParam))

    @property
    def params(self):
        if isinstance(self.command, device.general._SpecialDeviceCommandOneParam):
            return [self.command._opcode]
        elif isinstance(self.command, device.general._SpecialDeviceCommandTwoParam):
            return [self.command._instance, self.command._opcode]
        else:
            return []

    @property
    def instance(self):
        return self.pack_instance(self.command.instance)


class DaliResponse(DaliPayload):
    def __init__(self, query: command.Command, *args, **kwargs):
        super(DaliResponse, self).__init__(*args, **kwargs)
        self.query = query

    @property
    def is_dali2(self):
        return isinstance(self.query, device.general._DeviceCommand)

    @property
    def has_address(self):
        if isinstance(self.query, gear.general._GearCommand):
            return isinstance(self.query, (gear.general._StandardCommand, gear.general.DAPC))
        elif isinstance(self.query, device.general._DeviceCommand):
            return not isinstance(self.query, device.general._SpecialDeviceCommand)
        return False

    @property
    def address(self):
        return self.pack_address(self.query.destination)

    @property
    def instance(self):
        return self.pack_instance(self.query.instance)

    @property
    def has_instance(self): return isinstance(self.query, device.general._StandardInstanceCommand)

    @property
    def name(self) -> str:
        if self.query:
            return self.query.__class__.__name__
        return ''

    @property
    def mark(self):
        if self.query:
            if self.is_dali2:
                return Dali2Command.mark
            else:
                return DaliCommand.mark
        return 'response'

    @property
    def desc(self) -> str:
        if len(self.frame) == 0:
            return 'empty'
        elif self.query and self.query.response:
            _frame = frame.BackwardFrame(self.frame[0])
            resp = self.query.response(_frame)
            if isinstance(resp, (gear.general.QueryFadeTimeAndRateResponse, gear.general.QueryDeviceTypeResponse)):
                return str(resp)
            elif isinstance(resp, command.BitmapResponse):
                return bin(resp.value.as_integer)[2:].rjust(8, '0')
            elif isinstance(resp, command.YesNoResponse):
                return str(resp.value)
            else:
                return str(resp.value.as_integer)
        else:
            return super(DaliResponse, self).desc


class Dali2Event(DaliPayload):
    mark = 'event'

    def __init__(self, *args, **kwargs):
        super(Dali2Event, self).__init__(*args, **kwargs)
        self._event = None

    @property
    def has_address(self):
        return True

    @property
    def address(self):
        if isinstance(self.event.source1, address.Address):
            return self.pack_address(self.event.source1)
        else:
            assert isinstance(self.event.source1, address.Instance)
        return self.pack_instance(self.event.source1)

    @property
    def has_instance(self):
        return True

    @property
    def instance(self):
        return self.pack_instance(self.event.source2)

    @property
    def desc(self) -> str:
        return ' '.join([self.name, str(self.event.data.value)])

    @property
    def name(self) -> str:
        if isinstance(self.event.data, device.events.PresenceEvent):
            return 'Presence'

        if isinstance(self.event.data, device.events.LuminosityEvent):
            return 'Luminosity'

        if isinstance(self.event.data, device.events.ButtonEvent):
            return 'Button'

        return 'Event'

    @property
    def event(self) -> device.events.Event:
        if self._event is None:
            self._event = device.events.Event.of_frame(self.frame)

        return self._event

==> spread-5.9.0.59-py3-none-any.whl: spread_dali/payloads.py <==
from datetime import datetime

from spread_core.tools.utils import bytes2str
from . import command, frame, address, gear, device


class Payload:
    mark = 'Payload'

    def __init__(self, _frame: bytes):
        self.frame = _frame

    def __repr__(self):
        return f'[{self.name}]: {bytes2str(self.frame)}'

    def to_dict(self):
        return dict(
            time=self.time,
            hex=bytes2str(self.frame),
            name=self.name
        )

    @property
    def time(self):
        time = datetime.now()
        return time.strftime('%H:%M:%S') + f'.{str(int(time.microsecond / 1000)).ljust(3, "0")}'

    @property
    def name(self) -> str: return self.__class__.__name__

    @property
    def desc(self) -> str: return ''


class DaliPayload(Payload):
    @staticmethod
    def of(_frame: bytes, query_class=None):
        if len(_frame) < 2:
            return DaliResponse(query_class, _frame)
        elif len(_frame) == 2:
            return DaliCommand(_frame)
        elif len(_frame) == 3:
            if _frame[0] & 1 == 0:
                return Dali2Event(_frame)
            else:
                return Dali2Command(_frame)

        raise ValueError('frame size more than 3')

    @property
    def command(self) -> command.Command:
        if not hasattr(self, '_command'):
            self._command = command.from_frame(frame.ForwardFrame(8 * len(self.frame), self.frame),
                                               devicetype=self.last_device_type)
        return self._command

    @property
    def has_address(self):
        return False

    @property
    def has_instance(self):
        return False

    @property
    def address(self):
        raise ValueError("No address")

    @property
    def instance(self):
        raise ValueError("No instance")

    @property
    def last_device_type(self) -> int:
        return 0

    @staticmethod
    def pack_instance(inst) -> str:
        if isinstance(inst, address.InstanceNumber):
            return f'IN{inst._value}'
        elif isinstance(inst, address.InstanceType):
            return f'IT{inst._value}'
        elif isinstance(inst, address.InstanceGroup):
            return f'IG{inst._value}'
        elif isinstance(inst, address.InstanceBroadcast):
            return 'IB'
        elif isinstance(inst, address.FeatureInstanceType):
            return f'FT{inst._value}'
        elif isinstance(inst, address.FeatureInstanceGroup):
            return f'FG{inst._value}'
        elif isinstance(inst, address.FeatureInstanceNumber):
            return f'FN{inst._value}'
        elif isinstance(inst, address.FeatureDevice):
            return f'FD{inst._value}'
        elif isinstance(inst, address.FeatureInstanceBroadcast):
            return f'FB'
        return 'X3'

    @staticmethod
    def pack_address(addr) -> str:
        if isinstance(addr, address.Short):
            return f'D{addr.address}'
        elif isinstance(addr, address.Group):
            return f'G{addr.group}'
        elif isinstance(addr, address.Broadcast):
            return 'B'
        elif isinstance(addr, address.BroadcastUnaddressed):
            return 'BU'
        return 'X3'


class DaliCommand(DaliPayload):
    mark = 'dali'
    _last_device_type = 0

    @property
    def has_address(self):
        return isinstance(self.command, (gear.general._StandardCommand, gear.general.DAPC))

    @property
    def has_params(self):
        if isinstance(self.command, gear.general.DAPC):
            return True
        return isinstance(self.command, gear.general._StandardCommand) and self.command._hasparam

    @property
    def params(self):
        if isinstance(self.command, gear.general.DAPC):
            return [self.command.power]
        return [self.command.param]

    @property
    def address(self):
        return self.pack_address(self.command.destination)

    @property
    def desc(self) -> str:
        if self.has_params:
            return ' '.join([self.name, ' '.join([str(p) for p in self.params])])
        return self.name

    @property
    def name(self) -> str:
        return self.command.__class__.__name__

    @property
    def device_type(self) -> int:
        if isinstance(self.command, gear.general.EnableDeviceType):
            if len(self.params) > 0:
                return self.params[0]
        return 0

    @property
    def last_device_type(self) -> int:
        return self.__class__._last_device_type

    @last_device_type.setter
    def last_device_type(self, value: int):
        self.__class__._last_device_type = value


class Dali2Command(DaliCommand):
    mark = 'dali2'

    @property
    def has_address(self):
        return isinstance(self.command, device.general._DeviceCommand) and not isinstance(self.command, device.general._SpecialDeviceCommand)

    @property
    def has_instance(self):
        return isinstance(self.command, device.general._StandardInstanceCommand)

    @property
    def has_params(self):
        return isinstance(self.command, (
            device.general._SpecialDeviceCommandOneParam, device.general._SpecialDeviceCommandTwoParam))

    @property
    def params(self):
        if isinstance(self.command, device.general._SpecialDeviceCommandOneParam):
            return [self.command._opcode]
        elif isinstance(self.command, device.general._SpecialDeviceCommandTwoParam):
            return [self.command._instance, self.command._opcode]
        else:
            return []

    @property
    def instance(self):
        return self.pack_instance(self.command.instance)


class DaliResponse(DaliPayload):
    def __init__(self, query: command.Command, *args, **kwargs):
        super(DaliResponse, self).__init__(*args, **kwargs)
        self.query = query

    @property
    def is_dali2(self):
        return isinstance(self.query, device.general._DeviceCommand)

    @property
    def has_address(self):
        if isinstance(self.query, gear.general._GearCommand):
            return isinstance(self.query, (gear.general._StandardCommand, gear.general.DAPC))
        elif isinstance(self.query, device.general._DeviceCommand):
            return not isinstance(self.query, device.general._SpecialDeviceCommand)
        return False

    @property
    def address(self):
        return self.pack_address(self.query.destination)

    @property
    def instance(self):
        return self.pack_instance(self.query.instance)

    @property
    def has_instance(self): return isinstance(self.query, device.general._StandardInstanceCommand)

    @property
    def name(self) -> str:
        if self.query:
            return self.query.__class__.__name__
        return ''

    @property
    def mark(self):
        if self.query:
            if self.is_dali2:
                return Dali2Command.mark
            else:
                return DaliCommand.mark
        return 'response'

    @property
    def desc(self) -> str:
        if len(self.frame) == 0:
            return 'empty'
        elif self.query and self.query.response:
            _frame = frame.BackwardFrame(self.frame[0])
            resp = self.query.response(_frame)
            if isinstance(resp, (gear.general.QueryFadeTimeAndRateResponse, gear.general.QueryDeviceTypeResponse)):
                return str(resp)
            elif isinstance(resp, command.BitmapResponse):
                return bin(resp.value.as_integer)[2:].rjust(8, '0')
            elif isinstance(resp, command.YesNoResponse):
                return str(resp.value)
            else:
                return str(resp.value.as_integer)
        else:
            return super(DaliResponse, self).desc


class Dali2Event(DaliPayload):
    mark = 'event'

    def __init__(self, *args, **kwargs):
        super(Dali2Event, self).__init__(*args, **kwargs)
        self._event = None

    @property
    def has_address(self):
        return True

    @property
    def address(self):
        if isinstance(self.event.source1, address.Address):
            return self.pack_address(self.event.source1)
        else:
            assert isinstance(self.event.source1, address.Instance)
        return self.pack_instance(self.event.source1)

    @property
    def has_instance(self):
        return True

    @property
    def instance(self):
        return self.pack_instance(self.event.source2)

    @property
    def desc(self) -> str:
        return ' '.join([self.name, str(self.event.data.value)])

    @property
    def name(self) -> str:
        if isinstance(self.event.data, device.events.PresenceEvent):
            return 'Presence'

        if isinstance(self.event.data, device.events.LuminosityEvent):
            return 'Luminosity'

        if isinstance(self.event.data, device.events.ButtonEvent):
            return 'Button'

        return 'Event'

    @property
    def event(self) -> device.events.Event:
        if self._event is None:
            self._event = device.events.Event.of_frame(self.frame)

        return self._event
